@@ -4,6 +4,9 @@ import os
 import sys
 import fnmatch
 import types
+import traceback
+import inspect
+import pkgutil
 
 from .utils import AcceptHeader
 from .http import Response, Request
@@ -14,109 +17,92 @@ from .decorators import _property
 
 logger = logging.getLogger(__name__)
 
-_module_cache = {}
 
-def get_controllers(controller_prefix):
-    """get all the modules in the controller_prefix
+class Router(object):
 
-    returns -- set -- a set of string module names"""
-    global _module_cache
-    if not controller_prefix:
-        raise ValueError("controller prefix is empty")
-    if controller_prefix in _module_cache:
-        return _module_cache[controller_prefix]
+    _module_name_cache = {}
 
-    module = importlib.import_module(controller_prefix)
-    basedir = os.path.dirname(module.__file__)
-    modules = set()
+    @property
+    def controllers(self):
+        """get all the modules in the controller_prefix
 
-    for root, dirs, files in os.walk(basedir, topdown=True):
-        dirs[:] = [d for d in dirs if d[0] != '.' or d[0] != '_']
+        returns -- set -- a set of string module names"""
+        controller_prefix = self.controller_prefix
+        _module_name_cache = type(self)._module_name_cache # static cache, not instance
+        if controller_prefix in _module_name_cache:
+            return _module_name_cache[controller_prefix]
 
-        module_name = root.replace(basedir, '', 1)
-        module_name = [controller_prefix] + filter(None, module_name.split('/'))
-        for f in fnmatch.filter(files, '*.py'):
-            if f.startswith('__init__'):
-                modules.add('.'.join(module_name))
-            else:
-                # we want to ignore any "private" modules
-                if not f.startswith('_'):
-                    file_name = os.path.splitext(f)[0]
-                    modules.add('.'.join(module_name + [file_name]))
+        module = self.get_module(controller_prefix)
 
-    _module_cache.setdefault(controller_prefix, {})
-    _module_cache[controller_prefix] = modules
-    return modules
+        if hasattr(module, "__path__"):
+            # path attr exists so this is a package
+            modules = self.find_modules(module.__path__[0], controller_prefix)
 
+        else:
+            # we have a lonely .py file
+            modules = set([controller_prefix])
 
-class Call(object):
-    """
-    Where all the routing magic happens
+        _module_name_cache.setdefault(controller_prefix, {})
+        _module_name_cache[controller_prefix] = modules
 
-    we always translate an HTTP request using this pattern: METHOD /module/class/args?kwargs
-
-    GET /foo -> controller_prefix.version.foo.Default.get
-    POST /foo/bar -> controller_prefix.version.foo.Bar.post
-    GET /foo/bar/che -> controller_prefix.version.foo.Bar.get(che)
-    POST /foo/bar/che?baz=foo -> controller_prefix.version.foo.Bar.post(che, baz=foo)
-    """
-
-    controller_prefix = u""
-    """since endpoints interprets requests as /module/class, you can use this to do: controller_prefix.module.class"""
-
-    content_type = "application/json"
-    """the content type this call is going to represent"""
-
-    @_property
-    def request(self):
-        '''
-        Call.request, this request object is used to decide how to route the client request
-
-        a Request instance to be used to translate the request to a controller
-        '''
-        return Request()
-
-    @_property
-    def response(self):
-        '''
-        Call.response, this object is used to decide how to answer the client
-
-        a Response instance to be returned from handle populated with info from controller
-        '''
-        return Response()
-
-    def __init__(self, controller_prefix, *args, **kwargs):
-        '''
-        create the instance
-
-        controller_prefix -- string -- the module path where all your controller modules live
-        *args -- tuple -- convenience, in case you extend and need something in another method
-        **kwargs -- dict -- convenience, in case you extend
-        '''
-        if not controller_prefix:
-            raise ValueError("controller_prefix was empty")
+        return modules
 
+    def __init__(self, controller_prefix, path_args=None):
         self.controller_prefix = controller_prefix
-        self.args = args
-        self.kwargs = kwargs
+        if not controller_prefix:
+            raise ValueError("controller prefix is empty")
 
-    def get_controllers(self, controller_prefix):
-        """return a set of string controllers that includes controller_prefix and
-        any sub modules underneath it
+        if not path_args: path_args = []
+        self.load(path_args)
 
-        controller_prefix -- string -- you pass this in because it can be different
-            values (not just self.controller_prefix) depending on if a Versioned call
-            is being used, etc.
-        """
-        cset = get_controllers(controller_prefix)
-        return cset
+    def load(self, path_args):
+        self.controller_class_name = u"Default"
+        module_name, args = self.get_module_name(path_args)
+        self.controller_module_name = module_name
+        self.controller_module = self.get_module(module_name)
+
+        class_object = None
+        if args:
+            class_object = self.get_class(self.controller_module, args[0].capitalize())
+
+        if class_object:
+            args.pop(0)
+
+        else:
+            class_object = self.get_class(self.controller_module, self.controller_class_name)
+
+        if class_object:
+            self.controller_class_name = class_object.__name__
+        self.controller_class = class_object
+        self.controller_method_args = args
+
+    def find_modules(self, path, prefix):
+        """recursive method that will find all the submodules of the given module
+        at prefix with path"""
+
+        modules = set([prefix])
+
+        # https://docs.python.org/2/library/pkgutil.html#pkgutil.iter_modules
+        for module_info in pkgutil.iter_modules([path]):
+            # we want to ignore any "private" modules
+            if module_info[1].startswith('_'): continue
+
+            module_prefix = ".".join([prefix, module_info[1]])
+            if module_info[2]:
+                # module is a package
+                submodules = self.find_modules(os.path.join(path, module_info[1]), module_prefix)
+                modules.update(submodules)
+            else:
+                modules.add(module_prefix)
+
+        return modules
 
     def get_module_name(self, path_args):
         """returns the module_name and remaining path args.
 
         return -- tuple -- (module_name, path_args)"""
-        controller_prefix = self.get_normalized_prefix()
-        cset = self.get_controllers(controller_prefix)
+        controller_prefix = self.controller_prefix
+        cset = self.controllers
         module_name = controller_prefix
         mod_name = module_name
         while path_args:
@@ -143,6 +129,67 @@ class Call(object):
 
         return class_object
 
+
+class Call(object):
+    """
+    Where all the routing magic happens
+
+    we always translate an HTTP request using this pattern: METHOD /module/class/args?kwargs
+
+    GET /foo -> controller_prefix.version.foo.Default.get
+    POST /foo/bar -> controller_prefix.version.foo.Bar.post
+    GET /foo/bar/che -> controller_prefix.version.foo.Bar.get(che)
+    POST /foo/bar/che?baz=foo -> controller_prefix.version.foo.Bar.post(che, baz=foo)
+    """
+    router_class = Router
+
+    controller_prefix = u""
+    """since endpoints interprets requests as /module/class, you can use this to do: controller_prefix.module.class"""
+
+    content_type = "application/json"
+    """the content type this call is going to represent"""
+
+    charset = 'UTF-8'
+    """the default charset of the call, this will be passed down in the response"""
+
+    @_property
+    def version(self):
+        """
+        versioning is based off of this post 
+        http://urthen.github.io/2013/05/09/ways-to-version-your-api/
+        """
+        v = None
+        accept_header = self.request.get_header('accept', u"")
+        if accept_header:
+            if not self.content_type:
+                raise ValueError("You are versioning a call with no content_type")
+
+            a = AcceptHeader(accept_header)
+            for mt in a.filter(self.content_type):
+                v = mt[2].get("version", None)
+                if v: break
+
+        return v
+
+    def __init__(self, controller_prefix, *args, **kwargs):
+        '''
+        create the instance
+
+        controller_prefix -- string -- the module path where all your controller modules live
+        *args -- tuple -- convenience, in case you extend and need something in another method
+        **kwargs -- dict -- convenience, in case you extend
+        '''
+        if not controller_prefix:
+            raise ValueError("controller_prefix was empty")
+
+        self.controller_prefix = controller_prefix
+        self.args = args
+        self.kwargs = kwargs
+
+        self.router = None
+        self.request = None
+        self.response = None
+
     def get_kwargs(self):
         """combine GET and POST params to be passed to the controller"""
         req = self.request
@@ -168,42 +215,27 @@ class Call(object):
         d = {}
         req = self.request
         path_args = list(req.path_args)
-        d['module_name'] = u""
-        d['class_name'] = u"Default"
-        d['module'] = None
-        d['class'] = None
-        d['method'] = req.method.upper()
-        d['args'] = []
-        d['kwargs'] = {}
+        router = self.router_class(self.controller_prefix, path_args)
 
-        module_name, path_args = self.get_module_name(path_args)
-        d['module_name'] = module_name
-        d['module'] = self.get_module(d['module_name'])
+        d['module'] = router.controller_module
+        d['module_name'] = router.controller_module_name
 
-        class_object = None
-        if path_args:
-            class_object = self.get_class(d['module'], path_args[0].capitalize())
+        d['class'] = router.controller_class
+        d['class_name'] = router.controller_class_name
 
-        if class_object:
-            path_args.pop(0)
-
-        else:
-            class_object = self.get_class(d['module'], d['class_name'])
-            if not class_object:
-                class_object = None
-                raise TypeError(
-                    "could not find a valid controller with {}.{}.{}".format(
-                        d['module_name'],
-                        d['class_name'],
-                        d['method']
-                    )
-                )
-
-        d['class'] = class_object
-        d['class_name'] = class_object.__name__
-        d['args'] = path_args
+        d['method'] = self.get_normalized_method()
+        d['args'] = router.controller_method_args
         d['kwargs'] = self.get_kwargs()
 
+        if not d['class']:
+            raise TypeError(
+                "could not find a valid controller with {}.{}.{}".format(
+                    d['module_name'],
+                    d['class_name'],
+                    d['method']
+                )
+            )
+
         return d
 
     def get_callback(self, controller_info):
@@ -221,7 +253,8 @@ class Call(object):
                 controller_info['method'])
             )
 
-        except AttributeError, e:
+        except AttributeError as e:
+            logger.warning(str(e), exc_info=True)
             r = self.request
             raise CallError(405, "{} {} not supported".format(r.method, r.path))
 
@@ -237,10 +270,17 @@ class Call(object):
         try:
             d = self.get_controller_info()
 
-        except (ImportError, AttributeError, TypeError), e:
-            r = self.request
+        except IOError as e:
+            logger.warning(str(e), exc_info=True)
+            raise CallError(
+                408,
+                "The client went away before the request body was retrieved."
+            )
+
+        except (ImportError, AttributeError, TypeError) as e:
             exc_info = sys.exc_info()
             logger.warning(str(e), exc_info=exc_info)
+            r = self.request
             raise CallError(
                 404,
                 "{} not found because of {} \"{}\" on {}:{}".format(
@@ -262,79 +302,65 @@ class Call(object):
         """
         return self.controller_prefix
 
-    def generate_body(self):
-        """create a generator that returns the body for the given request
+    def get_normalized_method(self):
+        """
+        perform any normalization of the controller's method
 
-        this is wrapped like this so that a controller can use yield result and then
-        have it pick up right where it left off after resturning back to the client
-        what yield returned, it's just a tricksy way of being able to defer some
-        processing until after responding to the client
+        return -- string -- the full method name to be used
+        """
+        method = self.request.method.upper()
+        version = self.version
+        if version:
+            method += "_{}".format(version)
 
-        return -- generator -- a body generator"""
-        try:
-            body_generated = False
-            self.response.headers['Content-Type'] = self.content_type
-            callback, callback_args, callback_kwargs = self.get_callback_info()
-            body = self.handle_controller(callback, callback_args, callback_kwargs)
-            if isinstance(body, types.GeneratorType):
-                for b in body:
-                    body_generated = True
-                    yield b
-
-            else:
-                yield body
-
-        except Exception as e:
-            if body_generated:
-                raise
-            else:
-                body = self.handle_error(e)
-                yield body
+        return method
 
     def handle_controller(self, callback, callback_args, callback_kwargs):
         body = callback(*callback_args, **callback_kwargs)
         return body
 
-    def handle_error(self, e):
+    def handle_error(self, e, **kwargs):
         ret = None
         if isinstance(e, CallStop):
-            exc_info = sys.exc_info()
-            logger.info(str(e), exc_info=exc_info)
+            logger.info(str(e), exc_info=True)
             self.response.code = e.code
             #self.response.body = e.body
-            self.response.headers.update(e.headers)
+            self.response.add_headers(e.headers)
             ret = e.body
 
         elif isinstance(e, Redirect):
             #logger.exception(e)
-            exc_info = sys.exc_info()
-            logger.info(str(e), exc_info=exc_info)
+            logger.info(str(e), exc_info=True)
             self.response.code = e.code
             #self.response.body = None
-            self.response.headers.update(e.headers)
+            self.response.add_headers(e.headers)
             ret = None
 
         elif isinstance(e, (AccessDenied, CallError)):
             #logger.debug("Request Path: {}".format(self.request.path))
-            exc_info = sys.exc_info()
-            logger.warning(str(e), exc_info=exc_info)
+            logger.warning(str(e), exc_info=True)
             self.response.code = e.code
             #self.response.body = e
-            self.response.headers.update(e.headers)
+            self.response.add_headers(e.headers)
             ret = e
 
+        elif isinstance(e, NotImplementedError):
+            logger.warning(str(e), exc_info=True)
+            self.response.code = 501
+
         elif isinstance(e, TypeError):
-            # this is raised when there aren't enough args passed to controller
-            exc_info = sys.exc_info()
-            logger.warning(str(e), exc_info=exc_info)
-            self.response.code = 404
-            #self.response.body = e
-            ret = e
+            e_msg = unicode(e)
+            if e_msg.startswith(self.request.method) and 'argument' in e_msg:
+                logger.debug(e_msg, exc_info=True)
+                self.response.code = 404
+
+            else:
+                logger.exception(e)
+                self.response.code = 500
 
         else:
             logger.exception(e)
             self.response.code = 500
-            #self.response.body = e
             ret = e
 
         return ret
@@ -343,63 +369,29 @@ class Call(object):
         """returns a response where the controller is already evaluated
 
         return -- Response() -- the response object with a body already"""
-        self.ghandle()
-        for b in self.response.gbody: pass
+        body = None
+        callback = None
+        callback_args = []
+        callback_kwargs = {}
+        try:
+            self.response.set_header('Content-Type', "{};charset={}".format(self.content_type, self.charset))
+            self.response.charset = self.charset
+            callback, callback_args, callback_kwargs = self.get_callback_info()
+            body = self.handle_controller(callback, callback_args, callback_kwargs)
+
+        except Exception as e:
+            body = self.handle_error(
+                e,
+                callback_args=callback_args,
+                callback_kwargs=callback_kwargs,
+                callback=callback
+            )
+
+        finally:
+            self.response.body = body
+            if self.response.code == 204:
+                self.response.headers.pop('Content-Type', None)
+                self.response.body = None
+
         return self.response
 
-    def ghandle(self):
-        '''
-        return a response that is ready to have the controller evaluated, you can
-        trigger the controller being called and everything by just calling the body
-
-            response.body
-
-        or by iterating through the generator
-
-            for b in response.gbody: pass
-
-        return -- Response() -- the response object, ready to be populated with
-            information once the body is called
-        '''
-        self.response.gbody = self.generate_body()
-        return self.response
-
-
-class VersionCall(Call):
-    """
-    versioning is based off of this post: http://urthen.github.io/2013/05/09/ways-to-version-your-api/
-    """
-    default_version = None
-    """set this to the default version if you want a fallback version, if this is None then version check is enforced"""
-
-    def get_normalized_prefix(self):
-        cp = u""
-        if hasattr(self, "controller_prefix"):
-            cp = self.controller_prefix
-        v = self.get_version()
-        if cp:
-            cp += u".{}".format(v)
-        else:
-            cp = v
-
-        return cp
-
-    def get_version(self):
-        if not self.content_type:
-            raise ValueError("You are versioning a call with no content_type")
-
-        v = None
-        accept_header = self.request.get_header('accept', u"")
-        if accept_header:
-            a = AcceptHeader(accept_header)
-            for mt in a.filter(self.content_type):
-                v = mt[2].get(u"version", None)
-                if v: break
-
-        if not v:
-            v = self.default_version
-            if not v:
-                raise CallError(406, "Expected accept header with {};version=N media type".format(self.content_type))
-
-        return v
-
