@@ -1,3 +1,92 @@
+import os
+import re
+import mimetypes
+try:
+    import urlparse
+except ImportError:
+    import urllib.parse as urlparse #py3
+
+
+class Path(str):
+    def __new__(cls, s):
+        s = os.path.abspath(os.path.expanduser(str(s)))
+        return super(Path, cls).__new__(cls, s)
+
+
+class Host(str):
+    """A host string is defined as hostname:port for my purposes
+
+    This class attempts to sooth the rough edges to make sure there is access to
+    what is needed
+
+    example --
+        h = Host("https://foo.com:8080")
+        print h # foo.com:8080
+        print h.hostname # foo.com
+        print h.url # http://foo.com:8080
+        print h.port # 8080
+    """
+    @property
+    def hostname(self):
+        b = self.split(":")
+        return b[0]
+
+    @property
+    def port(self):
+        b = self.split(":")
+        return int(b[1]) if len(b) > 1 else 0
+
+    @property
+    def netloc(self):
+        return self
+
+    @property
+    def url(self):
+        return "{}://{}".format(self.scheme, self)
+
+    def __new__(cls, host):
+        b = urlparse.urlparse(host.rstrip("/"))
+        if b.scheme:
+            scheme = b.scheme.lower()
+            netloc = b.netloc
+
+        else:
+            scheme = "http"
+            netloc = b.path
+
+        netloc = netloc.lower()
+        instance = super(Host, cls).__new__(cls, netloc)
+
+        instance.scheme = scheme
+        return instance
+
+
+class MimeType(object):
+    """This is just a thin wrapper around Python's built-in MIME Type stuff
+
+    https://docs.python.org/2/library/mimetypes.html
+    """
+
+    @classmethod
+    def find_type(cls, val):
+        """return the mimetype from the given string value
+
+        if value is a path, then the extension will be found, if val is an extension then
+        that will be used to find the mimetype
+        """
+        mt = ""
+        index = val.rfind(".")
+        if index == -1:
+            val = "fake.{}".format(val)
+        elif index == 0:
+            val = "fake{}".format(val)
+
+        mt = mimetypes.guess_type(val)[0]
+        if mt is None:
+            mt = ""
+
+        return mt
+
 
 class AcceptHeader(object):
     """
@@ -89,7 +178,7 @@ class AcceptHeader(object):
         for x in self.__iter__():
             # all the params have to match to make the media type valid
             matched = True
-            for k, v in params.iteritems():
+            for k, v in params.items():
                 if x[2].get(k, None) != v:
                     matched = False
                     break
